package thkoeln.archilab.ecommerce.domainprimitives;

import javax.persistence.Embeddable;

import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PostalCodeType;



@Embeddable
@EqualsAndHashCode
@NoArgsConstructor
public class PersonalAddress implements PersonalAddressType {
    private String street;
    private String city;
    private PostalCode postalCode;

    public PersonalAddress(String street, String city, PostalCodeType postalCode) {
        validatePersonalAddress(street, city, postalCode);
        this.street = street;
        this.city = city;
        this.postalCode = (PostalCode)postalCode;
    }

    @Override
    public String getStreet() {
        return street;
    }

    @Override
    public String getCity() {
        return city;
    }

    @Override
    public PostalCode getPostalCode() {
        return postalCode;
    }


    public static void validatePersonalAddress(String street, String city, PostalCodeType postalCode) throws ShopException {
        if (street == null || street.isEmpty()) {
            throw new ShopException("Street cannot be null or empty");
        }
        if (city == null || city.isEmpty()) {
            throw new ShopException("City cannot be null or empty");
        }
        if (postalCode == null) {
            throw new ShopException("Postal code cannot be null");
        }
    }

    public static PersonalAddressType of(String street, String city, PostalCodeType postalCode)  {
        validatePersonalAddress(street, city, postalCode);
        return new PersonalAddress(street, city, postalCode);
    }
}