package thkoeln.archilab.ecommerce.domainprimitives;

import javax.persistence.Embeddable;

import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PostalCodeType;


@Embeddable
@EqualsAndHashCode
@NoArgsConstructor
public class PostalCode implements PostalCodeType {     //also with help of chatgpt

    private String postalCode;

    public PostalCode(String postalCode) {
        if(postalCode.length() < 1) throw new ShopException("Empty Postal Code");
        this.postalCode = postalCode;
    }

    @Override
    public String toString() {
        return postalCode;
    }

    @Override
    public int distance(PostalCodeType otherPostalCode) {
        if (otherPostalCode == null) {
            throw new ShopException("Other postal code cannot be null");
        }
        String other = otherPostalCode.toString();

        // Check if both postal codes are the same
        if (this.postalCode.equals(other)) {
            return 0;
        }

        // Calculate the imprecise distance
        int distance = 0;

        
        // Compare each digit from the right
        for (int i = 0; i < this.postalCode.length(); i++) {

            char thisDigit = this.postalCode.charAt(i);

            char otherDigit = other.charAt(i);


            if (thisDigit != otherDigit) {
                distance = this.postalCode.length() - i;
                break;
            }
        }



        // Handle special case for the first digit
        int firstDigitThis = Character.getNumericValue(this.postalCode.charAt(0));
        int firstDigitOther = Character.getNumericValue(other.charAt(0));
        if(firstDigitThis == firstDigitOther) return distance;

        int smallerRegionDistance = Math.abs(firstDigitOther - firstDigitThis +10 ) % 10;

        int regionDistance = Math.abs(firstDigitThis - firstDigitOther +10 ) % 10;

        if(smallerRegionDistance < regionDistance){
            distance += smallerRegionDistance;
        }else{
            distance += regionDistance;
        }

        return distance;
    }

    public static PostalCode of(String postalCodeAsString) {
        if (postalCodeAsString == null) {
            throw new ShopException("Postal code cannot be null");
        }
        if (!postalCodeAsString.matches("\\d{5}") || postalCodeAsString.endsWith("0000")) {
            throw new ShopException("Invalid postal code: " + postalCodeAsString);
        }
        return new PostalCode(postalCodeAsString);
    }
}
