package thkoeln.archilab.ecommerce.solution.depot.application.services;


import thkoeln.archilab.ecommerce.solution.depot.domain.entities.Depot;
import thkoeln.archilab.ecommerce.solution.depot.domain.entities.StockLevel;
import thkoeln.archilab.ecommerce.solution.depot.domain.repositories.DepotRepository;
import thkoeln.archilab.ecommerce.solution.product.application.DepotProductChecker;
import thkoeln.archilab.ecommerce.solution.product.application.ShoppingCartProductChecker;
import thkoeln.archilab.ecommerce.solution.product.domain.entities.Product;
import thkoeln.archilab.ecommerce.solution.product.domain.repositories.ProductRepository;
import thkoeln.archilab.ecommerce.usecases.DepotUseCases;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.domainprimitives.PersonalAddress;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


@Service //
public class DepotService implements DepotUseCases, DepotProductChecker { // warum wurde StockManagementUseCases so stumpf umbenannt?
    
    private final ProductRepository productRepo;
    private final DepotRepository depotRepo;

    //private final OrderProductChecker orderProductChecker;
    private final ShoppingCartProductChecker shoppingCartProductChecker;
    


    // , OrderProductChecker orderProductChecker
    //
    @Autowired
    public DepotService(ProductRepository productRepo  , DepotRepository depotRepo, ShoppingCartProductChecker shoppingCartProductChecker){

        this.productRepo = productRepo;
        //this.orderProductChecker = orderProductChecker;
        this.shoppingCartProductChecker = shoppingCartProductChecker;
        this.depotRepo = depotRepo;
    }



    
    @Override
    public int countProductInShoppingCarts(UUID productId) {
        return getAvailableStock(productId);
    }


    
    @Override
    public int getAvailableStock(UUID depotId, UUID productId) {

        Depot depot = depotRepo.findByDepotId(depotId);

          
        List<StockLevel> allProductsStockLevel = depot.getStockLevels(); //all products StockLevel
        for (StockLevel stockLevel : allProductsStockLevel) {
            
            if(stockLevel.getProduct().getProductId().equals(productId)){
                return stockLevel.getStock();
            }
        }
        

        return 0;
    }

    @Override
    public int getAvailableStock(UUID productId) {
        if (productId == null) throw new ShopException("getAvailableStock() Error, wrong input.");

        Product product = productRepo.findByProductId(productId);
        if(product == null) throw new ShopException("getAvailableStock() Error, Product does not exist.");

        List<Depot> depots = depotRepo.findAll();
        int totalStock = 0;

         
      /*   for (Depot depot : depots) {
                
            List<StockLevel> allProductsStockLevel = depot.getStockLevels(); //all products StockLevel
            for (StockLevel stockLevel : allProductsStockLevel) {
                
                if(stockLevel.getProduct().getProductId().equals(productId)){
                    totalStock += stockLevel.getStock();
                }
            }
        }*/
        //if(product.getTotalStock() != totalStock) throw new ShopException("Misha u did smth wrong");

        return product.getTotalStock();//+shoppingCartProductChecker.countProductInShoppingCarts(productId);
    }

    @Override
    public UUID addNewDepot(PersonalAddressType address, String name) {
        

        if(address == null || name == null || name.length() == 0) throw new ShopException("getAvailableStock() Error, wrong input.");

        Depot newDepot = new Depot();
        UUID outputUUID = UUID.randomUUID();
        newDepot.setDepotId(outputUUID);
        newDepot.setName(name);

        newDepot.setAddress((PersonalAddress)address);

        depotRepo.save(newDepot);
        return outputUUID;
    }

    @Override
    public void deleteAllDepots() {
        depotRepo.deleteAll();
    }

    @Override
    public void addToStock(UUID depotId, UUID productId, int addedQuantity) {
        
        if(depotId == null || productId == null || addedQuantity < 0) throw new ShopException("addToStock() Error, wrong input.");

        Depot depot = this.depotRepo.findByDepotId(depotId);
        if(depot == null) throw new ShopException("addToStock()1 Error, wrong input.");

        
        Product productToUpdate = productRepo.findByProductId(productId);
        if(productToUpdate == null) throw new ShopException("addToStock()3 Error, wrong input.");

        StockLevel stockLevel = depot.findProductInStock(productId);
        if(stockLevel== null) {
            stockLevel = new StockLevel();
            stockLevel.setProduct(productToUpdate);
            stockLevel.setStock(addedQuantity);
            depot.addOrUpdateStockLevels(stockLevel);

            productToUpdate.setTotalStock(productToUpdate.getTotalStock() + addedQuantity);//maybe here some errors lol
            productRepo.save(productToUpdate);
            depotRepo.save(depot);

            return;
        }

        stockLevel.setStock(stockLevel.getStock() + addedQuantity); 
        depot.addOrUpdateStockLevels(stockLevel); 
        productToUpdate.setTotalStock(productToUpdate.getTotalStock() + addedQuantity);


        productRepo.save(productToUpdate); 
        depotRepo.save(depot);
    }

    @Override
    public void removeFromStock(UUID depotId, UUID productId, int removedQuantity) {

        if(depotId == null || productId == null || removedQuantity < 0) throw new ShopException("removeFromStock()1 Error, wrong input.");

        if(removedQuantity == 0) return;

        Depot depot = this.depotRepo.findByDepotId(depotId);
        if(depot == null) throw new ShopException("removeFromStock()2 Error, wrong input.");

        StockLevel stockLevel = depot.findProductInStock(productId);
        if(stockLevel == null) throw new ShopException("removeFromStock()3 Error, wrong input.");

        Product productToUpdate = productRepo.findByProductId(productId);
        if(productToUpdate == null) throw new ShopException("removeFromStock()4 Error, wrong input.");
        
        


        int fullStock = this.getAvailableStock(productId);

        int newStock = stockLevel.getStock() - removedQuantity;
                if(newStock < 0) throw new ShopException("removeFromStock()5 Error, wrong input.");

        int amountInCarts = shoppingCartProductChecker.countProductInShoppingCarts(productId);

        if(fullStock-amountInCarts >= removedQuantity){
            stockLevel.setStock(newStock);
            depot.addOrUpdateStockLevels(stockLevel);
            depotRepo.save(depot);

            
            productToUpdate.setTotalStock(newStock);
            productRepo.save(productToUpdate);

            return;
        } 

        int newStockValue =  stockLevel.getStock()  - removedQuantity ;
        int amountToFree =  amountInCarts - newStockValue ;
        stockLevel.setStock(0); 

        
        productToUpdate.setTotalStock(0);
        

        if(amountToFree > 0){
            shoppingCartProductChecker.removeProductInShoppingCarts(productId, amountToFree);

        }/*else{
            stockLevel.setStock(removedQuantity);
            depot.addOrUpdateStockLevels(stockLevel);
            productToUpdate.setTotalStock(removedQuantity);
            
        }*/
        productToUpdate.setTotalStock(newStockValue);
        stockLevel.setStock(newStockValue);
        depot.addOrUpdateStockLevels(stockLevel);
        productRepo.save(productToUpdate);
        depotRepo.save(depot);
                
       
    }

    @Override
    public void changeStockTo(UUID depotId, UUID productId, int newTotalQuantity) {
        if(depotId == null || productId == null || newTotalQuantity < 0) throw new ShopException("removeFromStock() Error, wrong input.");

        Depot depot = this.depotRepo.findByDepotId(depotId);
        if(depot == null) throw new ShopException("changeStockTo() Error, wrong input.");

        StockLevel stockLevel = depot.findProductInStock(productId);
        if(stockLevel == null) throw new ShopException("changeStockTo() Error, wrong input.");

        Product productToUpdate = productRepo.findByProductId(productId);
        if(productToUpdate == null) throw new ShopException("changeStockTo() Error, wrong input.");
        
        
        int reserved = shoppingCartProductChecker.countProductInShoppingCarts(productId);

        int oldStock = this.getAvailableStock(productId);
        if(oldStock == newTotalQuantity) return;

        int amountToFree = reserved - newTotalQuantity; 

        if(amountToFree > 0){
            shoppingCartProductChecker.removeProductInShoppingCarts(productId, amountToFree);

        }

        productToUpdate.setTotalStock(newTotalQuantity);
        stockLevel.setStock(newTotalQuantity);
        depot.addOrUpdateStockLevels(stockLevel);

        productRepo.save(productToUpdate);
        depotRepo.save(depot);
    

        
        
    }





    @Override
    public void removeFromStockChecker(UUID depotId, UUID productId, int removedQuantity) {
        removeFromStock(depotId, productId, removedQuantity);
    }


    /* 

    public void addToStock(UUID productId, int addedQuantity) {

        List<Depot> depots = depotRepo.findAll();
        if(depots == null || depots.size() < 1)throw new ShopException("addToStock() Error, no depots");

        Depot firstDepot = depots.get(0);
        //firstDepot. set stock 
    }


    public int getDepotWhichCoversCartProduct(UUID productId) {
        if (productId == null) throw new ShopException("getAvailableStock() Error, wrong input.");

        Product product = productRepo.findByProductId(productId);
        if(product == null) throw new ShopException("getAvailableStock() Error, Product does not exist.");

        List<Depot> depots = depotRepo.findAll();
        int totalStock = 0;

        for (Depot depot : depots) {
                
            List<StockLevel> allProductsStockLevel = depot.getStockLevel(); //all products StockLevel
            for (StockLevel stockLevel : allProductsStockLevel) {
                
                if(stockLevel.getProductId().equals(productId)){
                    totalStock += stockLevel.getStock();
                }
            }
        }

        return totalStock;//+shoppingCartProductChecker.countProductInShoppingCarts(productId);
    }

*/



/*
 * 
 *
    //@Override
    public void addToStock(UUID productId, int addedQuantity) {
        if(productId == null || addedQuantity < 0) throw new ShopException("addToStock() Error");
        if(addedQuantity == 0) return;

        Product product = productRepo.findByProductId(productId);
        if(product == null) throw new ShopException("addToStock() Error, Product does not exist.");

        int newStock = getAvailableStock(productId) + addedQuantity;
        product.setStock(newStock);
    }

    //@Override
    public void removeFromStock(UUID productId, int removedQuantity) {
        if(productId == null || removedQuantity < 0) throw new ShopException("removeFromStock() Error"); 

        Product product = productRepo.findByProductId(productId);
        if(product == null) throw new ShopException("removeFromStock() Error, Product does not exist.");

        
        int fullStock = this.getAvailableStock(productId);//product.getStock();
        //if(fullStock == removedQuantity) return;

        //List<ShoppingCart> carts = cartRepo.findAll();


       // int reservedProducts = productReference.countProductUsed(productId);


        if(product.getStock() >= removedQuantity){
            product.setStock(product.getStock() - removedQuantity);
            return;
        } 
        else if(fullStock < removedQuantity){
            throw new ShopException("removeFromStock() removing too much from stock results in negative stock");
        }

        int amountToFree = removedQuantity - product.getStock(); 
        product.setStock(0); 
        if(amountToFree > 0){
            shoppingCartProductChecker.removeProductInShoppingCarts(productId, amountToFree);

        }else{
            product.setStock(removedQuantity);
        }
        
        productRepo.save(product);



        /*

List<ShoppingCart> carts = cartRepo.findAll();

        if(product.getStock() > removedQuantity){
            product.setStock(product.getStock() - removedQuantity);
            return;
        } 
        else if(fullStock < removedQuantity){
            throw new ShopException("removeFromStock() removing too much from stock results in negative stock");
        }

        
        int amountToFree = removedQuantity - product.getStock(); 
        product.setStock(0);
        if(amountToFree > 0){
            for (ShoppingCart cart : carts) {
                List<OrderPart> orderParts = cart.getCartItems();
                
                for (OrderPart orderPart : orderParts) {
                    if(orderPart.getProduct().getProductId() == productId){  

                        int orderPartQuantity = orderPart.getQuantity();
                        if(orderPartQuantity- amountToFree > 0){
                            orderPart.setQuantity(orderPartQuantity- amountToFree);
                            amountToFree = 0;
                            break;
                        }else{
                            orderPart.setQuantity(0);
                            amountToFree = amountToFree - orderPartQuantity;
                        }
                    }
                }
                
                if(amountToFree == 0) break;
            }
        }
        //product.setStock(removedQuantity);
        *
    }

   // @Override
    public void changeStockTo(UUID productId, int newTotalQuantity) {
        if(productId == null || newTotalQuantity < 0) throw new ShopException("changeStockTo() Error"); 

        Product product = productRepo.findByProductId(productId);
        if(product == null) throw new ShopException("changeStockTo() Error, Product does not exist.");

        
        int fullStock = this.getAvailableStock(productId);//product.getStock();
        if(fullStock == newTotalQuantity) return;


        int amountToFree = fullStock - newTotalQuantity-product.getStock(); 
        if(amountToFree > 0){
            shoppingCartProductChecker.removeProductInShoppingCarts(productId, amountToFree);

        }else{
            product.setStock(newTotalQuantity);
        }
        
        


        
       /*  List<ShoppingCart> carts = cartRepo.findAll();

        int amountToFree = fullStock - newTotalQuantity-product.getStock(); 
        if(amountToFree > 0){
            for (ShoppingCart cart : carts) {
                
               List<OrderPart> orderParts = cart.getCartItems();
                
                for (OrderPart orderPart : orderParts) {
                    if(orderPart.getProduct().getProductId() == productId){  

                        int orderPartQuantity = orderPart.getQuantity();
                        if(orderPartQuantity- amountToFree > 0){
                            orderPart.setQuantity(orderPartQuantity- amountToFree);
                            amountToFree = 0;
                            break;
                        }else{
                            orderPart.setQuantity(0);
                            amountToFree = orderPartQuantity - amountToFree;
                        }
                    }
                }
                if(amountToFree == 0) break;
            }
        }
        product.setStock(newTotalQuantity);
    }

  */


}