package thkoeln.archilab.ecommerce.solution.order.application;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import thkoeln.archilab.ecommerce.solution.order.domain.Order;
import thkoeln.archilab.ecommerce.solution.order.domain.OrderRepository;
import thkoeln.archilab.ecommerce.solution.order.domain.valueObjects.orderPart.domain.OrderPart;
import thkoeln.archilab.ecommerce.solution.product.application.OrderProductChecker;
import thkoeln.archilab.ecommerce.solution.user.domain.entities.User;
import thkoeln.archilab.ecommerce.solution.user.domain.repositories.UserRepository;
import thkoeln.archilab.ecommerce.usecases.OrderUseCases;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.domainprimitives.EmailAddress;


@Service
public class OrderService implements OrderUseCases , OrderProductChecker{    

    private final OrderRepository orderRepo;
    private final UserRepository userRepo;

    @Autowired
    public OrderService(OrderRepository orderRepo, UserRepository userRepo){
        this.orderRepo = orderRepo;
        this.userRepo = userRepo;
    }

    



    @Override
    public Map<UUID, Integer> getOrderHistory(EmailAddressType userEmailAddress) {
        if(userEmailAddress == null ) throw new ShopException("getOrderHistory() Input Exception");
        
        User user = userRepo.findByEmailAddress((EmailAddress)EmailAddress.of(userEmailAddress.toString()));
        if (user == null) throw new ShopException("getOrderHistory() User does not exist Exception");

        //ArrayList<Order> orders = (ArrayList<Order>) orderRepo.findByUserEmailAddressAndStatus(user.getEmailAddress(), Order.OrderStatus.DELIVERED);  //only reserved status can be edited
        ArrayList<Order> orders = (ArrayList<Order>) orderRepo.findByUser(user); 
       
        HashMap<UUID, Integer> output = new HashMap<UUID, Integer>();
       
        for (Order order : orders) { 
            List<OrderPart> orderParts = order.getOrderParts();
            for (OrderPart orderPart : orderParts) {
                if(!output.containsKey(orderPart.getProduct().getProductId())){                                  
                    output.put(orderPart.getProduct().getProductId(), orderPart.getQuantity());                                   
                }
                else{
                    Integer amountOfProductOrdered = output.get(orderPart.getProduct().getProductId());                                   
                    output.put(orderPart.getProduct().getProductId(), amountOfProductOrdered + orderPart.getQuantity()); //Override value in map with new val
                }
            }
        }
        return output;
    }


    @Override
    public void deleteAllOrders() {
        orderRepo.deleteAll();  //orderPart must be cascading - so i dont need to delete them manually
    }



    @Override
    public boolean isProductInOrder(UUID productId) {
        List<Order> allOrders = orderRepo.findAll();
        
        for (Order order : allOrders) {
            List<OrderPart> orderItems = order.getOrderParts();
            for (OrderPart orderItem : orderItems) {
                if(orderItem.getProduct().getProductId().equals(productId)){
                    return true;
                }
            }
        }

        return false;
    }




   

}
