package thkoeln.archilab.ecommerce.solution.shipmentunit.application;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

import org.hibernate.cfg.NotYetImplementedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import thkoeln.archilab.ecommerce.solution.order.application.OrderService;
import thkoeln.archilab.ecommerce.solution.order.domain.Order;
import thkoeln.archilab.ecommerce.solution.order.domain.OrderRepository;
import thkoeln.archilab.ecommerce.solution.shoppingcart.application.CartPositionDto;
import thkoeln.archilab.ecommerce.solution.shoppingcart.application.ShoppingCartDto;
import thkoeln.archilab.ecommerce.solution.shoppingcart.application.ShoppingCartDtoMapperHelper;
import thkoeln.archilab.ecommerce.solution.shoppingcart.application.exceptions.ShoppingCartMethodNotAllowedException;
import thkoeln.archilab.ecommerce.solution.shoppingcart.application.exceptions.ShoppingCartNotFoundException;
import thkoeln.archilab.ecommerce.solution.shoppingcart.domain.entities.ShoppingCart;


@RestController
@Transactional
public class ShipmentUnitController {


    private final ShipmentUnitService shipmentUnitService;
//    private final ShipmentUnitRepository unitRepo;
    private final OrderRepository orderRepo;
    private final OrderService orderService;


    @Autowired
    public ShipmentUnitController( OrderRepository orderRepo, OrderService orderService, ShipmentUnitService shipmentUnitService){
        this.shipmentUnitService = shipmentUnitService;
//        this.unitRepo = unitRepo;
        this.orderRepo = orderRepo;
        this.orderService = orderService;
    }


        
    @GetMapping("/shipmentUnits")
    public ResponseEntity<List<ShipmentUnitDto>> getShipmentUnitByOrderId(@RequestParam(required = false) String orderId) 
    {

        
        if(orderId == null || orderId == ""){throw new ResponseStatusException(HttpStatus.METHOD_NOT_ALLOWED);}

        Optional<Order> order;

        try {
            order = orderRepo.findByOrderId(UUID.fromString(orderId));
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.METHOD_NOT_ALLOWED);
        }

        order = orderRepo.findByOrderId(UUID.fromString(orderId));

        if (order.isEmpty()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND);
        }


        List<UUID> depotIds = shipmentUnitService.getContributingDepotsForOrder(order.get().getOrderId());
        
        List<ShipmentUnitDto> output = new ArrayList<ShipmentUnitDto>() ;

        for (UUID depotId : depotIds) {
            ShipmentUnitDto toAdd = new ShipmentUnitDto();

            toAdd.setId(UUID.randomUUID());
            toAdd.setDepotId(depotId);
            toAdd.setOrderId(UUID.fromString(orderId));


            Map<UUID, Integer> unitParts = shipmentUnitService.getShipmentUnitForOrderAndDepot(UUID.fromString(orderId), depotId);
            

            List<CartPositionDto> shipmentUnitParts = new ArrayList<CartPositionDto>();

            for (Map.Entry<UUID, Integer> entry : unitParts.entrySet()) {
                CartPositionDto posToAdd = new CartPositionDto();
                posToAdd.setProductId(entry.getKey());
                posToAdd.setQuantity(entry.getValue());
                shipmentUnitParts.add(posToAdd);
            }
            toAdd.setShipmentUnitParts(shipmentUnitParts);

            output.add(toAdd);
        }


        return ResponseEntity.ok(output); 
    }




///shipmentUnits?orderId=... get
       
//    @GetMapping("/shipmentUnits")
//    public ResponseEntity<List<ShipmentUnitDto>> getOneShipmentUnitsByOrderId(@RequestParam(required = false) String orderId)
//            throws ShoppingCartMethodNotAllowedException {
//
//
//        if(orderId == null || orderId == ""){throw new ShoppingCartMethodNotAllowedException("No User with userId = " + orderId);}
//
//        List<ShipmentUnit> units;
//        try {
//            Optional<Order> tmpOrder = orderRepo.findById(UUID.fromString(orderId));
//
//            units = unitRepo.findByOrder(tmpOrder.get());
//        } catch (Exception e) {
//            throw new ShoppingCartNotFoundException(e.getMessage());
//        }
//
//        List<ShipmentUnitDto> unitDto = new ArrayList<ShipmentUnitDto>();
//        for (ShipmentUnit unit : units) {
//            ShipmentUnitDto tmp = ShipmentUnitDtoMapperHelper.mapToDto(unit);
//            unitDto.add(tmp);
//        }
//
//
//        return ResponseEntity.ok(unitDto);
//    }






   /*  @GetMapping
    public ResponseEntity<List<ShipmentUnitDto>> getShipmentUnitsByOrderId(@RequestParam(required = false) UUID orderId) {
        if (orderId == null) {
           // throw new ForbiddenException("Access to all shipment units is forbidden.");
        }

       // List<ShipmentUnitDto> shipmentUnits = shipmentUnitService.getShipmentUnitsByOrderId(orderId);
        return ResponseEntity.ok(shipmentUnits);
    }*/
}