package thkoeln.archilab.ecommerce.solution.user.application;

import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.client.RestClientTest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import thkoeln.archilab.ecommerce.domainprimitives.EmailAddress;
import thkoeln.archilab.ecommerce.solution.user.application.exceptions.UserMethodNotAllowedException;
import thkoeln.archilab.ecommerce.solution.user.application.exceptions.UserNotFoundException;
import thkoeln.archilab.ecommerce.solution.user.domain.entities.User;

@RestController
@Transactional
public class UserController {
    

     private final UserService userService;

    @Autowired
    public UserController(UserService userService ){
        this.userService = userService;
    }


    @GetMapping("/users")
    public ResponseEntity<UserDto> getOneUser(@RequestParam(required = false) String emailAddress)
    throws UserMethodNotAllowedException{

        if(emailAddress == null || emailAddress.length() == 0) throw new UserMethodNotAllowedException("getAllUsers is not allowed");
        
        User user;
        try{
            user = (User)userService.getUserData(EmailAddress.of(emailAddress));
        }catch(Exception e){
            throw new UserNotFoundException(e.getMessage());
        }
        
        UserDto userDto = UserDtoMapperHelper.mapToDto(user);

        return ResponseEntity.ok(userDto);
    }

    @GetMapping("/users/{id}")
    public ResponseEntity<UserDto> getOneUser(@PathVariable(required = false) UUID userId)
    throws UserMethodNotAllowedException{

        if(userId == null ) throw new UserNotFoundException("User with id " + userId + " not found");
        
        User user;
        try{
            user = (User)userService.getUserData(userId);
        }catch(Exception e){
            throw new UserNotFoundException(e.getMessage());
        }
        
        UserDto userDto = UserDtoMapperHelper.mapToDto(user);

        return ResponseEntity.ok(userDto);
    }

}
