package thkoeln.archilab.ecommerce.solution.user.application;

import thkoeln.archilab.ecommerce.domainprimitives.EmailAddress;
import thkoeln.archilab.ecommerce.domainprimitives.PersonalAddress;
import thkoeln.archilab.ecommerce.domainprimitives.PostalCode;
import thkoeln.archilab.ecommerce.domainprimitives.dto.EmailAddressDto;
import thkoeln.archilab.ecommerce.domainprimitives.dto.PersonalAddressDto;
import thkoeln.archilab.ecommerce.domainprimitives.dto.PostalCodeDto;
import thkoeln.archilab.ecommerce.solution.user.application.exceptions.UserCreationException;
import thkoeln.archilab.ecommerce.solution.user.domain.entities.User;

import org.hibernate.cfg.NotYetImplementedException;
import org.modelmapper.ModelMapper;

public final class UserDtoMapperHelper {

    private static ModelMapper modelMapper = new ModelMapper();

    public static UserDto mapToDto(User user) {
        return new UserDto(
            user.getId(),
            user.getName(),
            mapToEmailAddressDto(user.getEmailAddress()),
            mapToPersonalAddressDto(user.getAddress())
            
        );
    }

    private static EmailAddressDto mapToEmailAddressDto(EmailAddress emailAddress) {
        if (emailAddress == null) {
            return null;
        }
        return new EmailAddressDto(emailAddress.toString());
    }

    private static PersonalAddressDto mapToPersonalAddressDto(PersonalAddress address) {
        if (address == null) {
            return null;
        }
        return new PersonalAddressDto(
            address.getStreet(),
            address.getCity(),
            mapToPostalCodeDto(address.getPostalCode())
        );
    }

    private static PostalCodeDto mapToPostalCodeDto(PostalCode postalCode) {
        if (postalCode == null) {
            return null;
        }
        return new PostalCodeDto(postalCode.toString());
    }
    
}
