package thkoeln.archilab.ecommerce.solution.user.application;

import thkoeln.archilab.ecommerce.solution.user.domain.entities.User;
import thkoeln.archilab.ecommerce.solution.user.domain.entities.UserShoppingCartCreator;
import thkoeln.archilab.ecommerce.solution.user.domain.repositories.UserRepository;
//import thkoeln.archilab.ecommerce.solution.user.domain.valueObjects.personalAddress.domain.PersonalAddress;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.domainprimitives.EmailAddress;
import thkoeln.archilab.ecommerce.domainprimitives.PersonalAddress;
import thkoeln.archilab.ecommerce.domainprimitives.PostalCode;
import thkoeln.archilab.ecommerce.usecases.UserRegistrationUseCases;
import thkoeln.archilab.ecommerce.usecases.UserType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;

import java.util.Optional;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


@Service
public class UserService implements UserRegistrationUseCases {

    private final UserRepository userRepo;
    private final UserShoppingCartHelper cartHelper;
    private final UserShoppingCartCreator cartCreator;

    @Autowired
    public UserService(UserRepository userRepo , UserShoppingCartHelper cartHelper, UserShoppingCartCreator cartCreator){
        this.userRepo = userRepo;
        this.cartHelper = cartHelper;
        this.cartCreator = cartCreator;
    }

    

  

    @Override
    //public void register( String name, String emailAddress, String street, String city, String postalCode ){
    public void register(String name, EmailAddressType emailAddress, PersonalAddressType personalAddress) {
        
        if(emailAddress == null || emailAddress.toString().isEmpty() || name == null || name.isEmpty() || personalAddress == null 
        || personalAddress.getStreet() == null || personalAddress.getStreet().isEmpty() || personalAddress.getCity() == null || personalAddress.getCity().isEmpty()
        || personalAddress.getPostalCode() == null || personalAddress.getPostalCode().toString().isEmpty()  ){
            throw new ShopException("register() Register Input Data Validation failed");
        }
        
        if(userRepo.findByEmailAddress((EmailAddress)EmailAddress.of(emailAddress.toString())) != null){
            throw new ShopException("EMail already exists");
        }
        

        User newUser = new User(name, (EmailAddress)EmailAddress.of(emailAddress.toString()), 
        (PersonalAddress)PersonalAddress.of(personalAddress.getStreet(),personalAddress.getCity(),
        (PostalCode)personalAddress.getPostalCode()), UUID.randomUUID() );
        userRepo.save(newUser);

       
        cartCreator.createShoppingCartForUser(newUser);
    }


    @Override
    public void changeAddress( EmailAddressType userEmailAddress, PersonalAddressType personalAddress ){
        
        if(userEmailAddress == null || userEmailAddress.toString().isEmpty() || personalAddress == null 
        || personalAddress.getStreet() == null || personalAddress.getStreet().isEmpty() || personalAddress.getCity() == null || personalAddress.getCity().isEmpty() 
        || personalAddress.getPostalCode() == null || personalAddress.getPostalCode().toString().isEmpty()){
            throw new ShopException("changeAddress() Register Input Data Validation failed");
        }

        User userToUpdate = userRepo.findByEmailAddress((EmailAddress)EmailAddress.of(userEmailAddress.toString()));
        if(userToUpdate == null){
            throw new ShopException("EMail does not exist");
        }

       /*  PersonalAddress addressToUpdate = userToUpdate.getAddress();
        addressToUpdate.setStreet(personalAddress.getStreet()); 
        addressToUpdate.setCity(personalAddress.city());
        addressToUpdate.setPostalCode(postalCode);*/

        userToUpdate.setAddress( (PersonalAddress)PersonalAddress.of(personalAddress.getStreet(),personalAddress.getCity(),(PostalCode)personalAddress.getPostalCode()) );

        userRepo.save(userToUpdate);
    }

    @Override
    public UserType getUserData( EmailAddressType userEmailAddress ){
        if(userEmailAddress == null ){  //|| userEmailAddress.isEmpty()
            return null;    //Wrong input
        }

        User user = userRepo.findByEmailAddress((EmailAddress)EmailAddress.of(userEmailAddress.toString()));
        if(user == null){
            throw new ShopException("User does not exist");
        }

        return user;
    }

     public UserType getUserData( UUID id ){
        if(id == null ){  //|| userEmailAddress.isEmpty()
            return null;    //Wrong input
        }

        Optional<User> user = userRepo.findById(id);
        if(user == null || user.isEmpty()){
            throw new ShopException("E-Mail does not exist");
        }

        return user.get();
    }


    
    @Override
    public void deleteAllUsers(){
        //ArrayList<User> users = (ArrayList<User>) userRepo.findAll();

       // if(users != null && !users.isEmpty()) {
            //clear shoppingcarts manually cause cascade wont work kinda
            cartHelper.deleteAllCarts();
            
            userRepo.deleteAll(); 
      //  }
        
        
    }



}
