package thkoeln.archilab.ecommerce.usecases;

import thkoeln.archilab.ecommerce.ShopException;

import java.util.List;
import java.util.Map;
import java.util.UUID;


/**
 * This interface contains methods to query shipment units belonging to orders.
 * We assume that there is an algorithm in place that has calculated the most cost-efficient
 * setup of shipment units for a given order.
 */
public interface ShipmentUnitUseCases {

    /**
     * Query which depots are involved in the shipment of a given order.
     * @param orderId
     * @return A list of depots that are involved in the shipment of the order.
     *         The list will contain at least one depot.
     * @throws ShopException if ...
     *     - orderId is null
     *     - the order with that id does not exist
     */
    public List<UUID> getContributingDepotsForOrder( UUID orderId );


    /**
     * Query the content of the shipment units for a given order from a given depot.
     * @param orderId
     * @param depotId
     * @return A map with product IDs as keys, and the amount of that products contained in the
     *         shipment unit as values. If the depot is not involved in the
     *         shipment of the order, the map will be empty.
     * @throws ShopException if ...
     *      - orderId is null, or the order with that id does not exist
     *      - depotId is null, or the depot with that id does not exist
     */
    public Map<UUID, Integer> getShipmentUnitForOrderAndDepot(
            UUID orderId, UUID depotId );


    /**
     * Just for testing purposes. Deletes all shipment units.
     */
    public void deleteAllShipmentUnits();
}
