package thkoeln.archilab.ecommerce.usecases;

import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;

/**
 * This interface contains methods needed in the context of the registration of a user.
 */
public interface UserRegistrationUseCases {
    /**
     * Registers a new user
     *
     * @param name
     * @param emailAddress
     * @param personalAddress
     * @throws ShopException if ...
     *      - name is null or empty
     *      - emailAddress is null
     *      - user with the given email address already exists
     *      - personalAddress is null
     */
    public void register( String name, EmailAddressType emailAddress, PersonalAddressType personalAddress );


    /**
     * Changes the personalAddress of a user
     *
     * @param userEmailAddress
     * @param personalAddress
     * @throws ShopException if ...
     *      - emailAddress is null
     *      - user with the given email address does not exist
     *      - personalAddress is null
     */
    public void changeAddress( EmailAddressType userEmailAddress, PersonalAddressType personalAddress );


    /**
     * Returns the data of a user (by an object implementing UserType)
     * @param userEmailAddress
     * @return the user data
     * @throws ShopException if ...
     *      - emailAddress is null
     *      - the user with the given email address does not exist
     */
    public UserType getUserData( EmailAddressType userEmailAddress );



    /**
     * Clears all users, including all orders and shopping carts
     */
    public void deleteAllUsers();
}
