package thkoeln.archilab.ecommerce.usecases.domainprimitivetypes;

import thkoeln.archilab.ecommerce.ShopException;

public interface EmailAddressType {
    /**
     * @return the email address as a string
     */
    public String toString();


    /**
     * A special kind of "copy constructor": Returns a new email address object with
     * the same identifyer (the part left of the "@" sign) as the current one, but
     * with a new domain part (right of the "@" sign).
     * @param domainString - the new domain for the copied email address
     * @return the new email address
     * @throws ShopException if ...
     *     - domainString is null
     *     - the new email address would not be valid (see `of(...)` method)
     */
    public EmailAddressType sameIdentifyerDifferentDomain( String domainString );

    /**
     * Another special kind of "copy constructor": Returns a new email address object
     * with the same domain (the part right of the "@" sign) as the current one, but
     * with a new identifyer part (left of the "@" sign).
     * @param identifyerString - the new identifyer for the copied email address
     * @return the new email address
     * @throws ShopException if ...
     *     - identifyerString is null
     *     - the new email address would not be valid (see `of(...)` method)
     */
    public EmailAddressType sameDomainDifferentIdentifyer( String identifyerString );


    /**
     * Unfortunately, Java interfaces cannot contain static methods. However, we expect the
     * implementing class to provide a static factory method (simply named "of(...)"),
     * which creates an email address, given as a string.
     * We specify this factory method here as a comment, using the Javadoc documentation style.
     *
     * @param emailAddressAsString - the email address as a string.
     *      We will use a much simplified validation method to check if the email address is valid:
     *      - it must contain exactly one '@' character.
     *      - the part before the '@' and the part after the '@' must not be empty, contain of
     *        at least one of these characters (A..Z, a..z, or 0..9) and must not contain any whitespace characters
     *      - the parts before and after the '@' may contain one or several '.' as separators
     *      - two '.' characters must not be directly next to each other (so "test@this..example.com" is invalid)
     *      - the part after the '@' must end with ".de", ".at", ".ch", ".com", ".org"
     *        (for simplicity we do not allow any other domains)
     * @return a new EmailAddressType object matching the given email address
     * @throws ShopException if ...
     *      - emailAddressAsString is null
     *      - emailAddressAsString is not a valid email address (see above)
     */
     // public static EmailAddressType of( String emailAddressAsString );
}
