package thkoeln.archilab.ecommerce.usecases.masterdata;

import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Profile;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;
import thkoeln.archilab.ecommerce.usecases.*;

@Component
@Slf4j
@Profile("!test")
@SuppressWarnings("PMD")
public class StartupListener implements ApplicationListener<ContextRefreshedEvent>  {
    private ProductAndStockMasterDataInitializer productAndStockMasterDataInitializer;
    private UserMasterDataInitializer userMasterDataInitializer;

    private UserRegistrationUseCases userRegistrationUseCases;
    private ProductCatalogUseCases productCatalogUseCases;
    private DepotUseCases depotUseCases;
    private Purgatory purgatory;

    @Autowired
    public StartupListener( UserRegistrationUseCases userRegistrationUseCases,
                            ProductCatalogUseCases productCatalogUseCases,
                            DepotUseCases depotUseCases,
                            Purgatory purgatory ) {
        this.userRegistrationUseCases = userRegistrationUseCases;
        this.productCatalogUseCases = productCatalogUseCases;
        this.depotUseCases = depotUseCases;
        this.purgatory = purgatory;
        productAndStockMasterDataInitializer = new ProductAndStockMasterDataInitializer(
                productCatalogUseCases, depotUseCases );
        userMasterDataInitializer = new UserMasterDataInitializer( userRegistrationUseCases );
    }

    @Override
    public void onApplicationEvent( ContextRefreshedEvent contextRefreshedEvent ) {
        log.info( "StartupListener initializing master data ..." );
        purgatory.deleteEverything();
        userMasterDataInitializer = new UserMasterDataInitializer( userRegistrationUseCases );
        userMasterDataInitializer.registerAllUsers();

        productAndStockMasterDataInitializer = new ProductAndStockMasterDataInitializer(
                productCatalogUseCases, depotUseCases );
        productAndStockMasterDataInitializer.addAllProducts();
        productAndStockMasterDataInitializer.addAllDepots();
        productAndStockMasterDataInitializer.addAllStock();
    }
}
