package thkoeln.archilab.ecommerce.usecases.masterdata;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.UserRegistrationUseCases;
import thkoeln.archilab.ecommerce.usecases.UserType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;

import java.util.Random;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * This is a test helper class that initializes and registers users in the system,
 * using the given interface(s).
 */
@Component
@Slf4j
@SuppressWarnings("PMD")
public class UserMasterDataInitializer {

    private UserRegistrationUseCases userRegistrationUseCases;
    private Random random = new Random();

    public static final String EUR = "EUR";

    @Autowired
    public UserMasterDataInitializer( UserRegistrationUseCases userRegistrationUseCases ) {
        this.userRegistrationUseCases = userRegistrationUseCases;
    }

    public final static String[] USER_NAME = new String[]{
            "Max Müller",
            "Sophie Schmitz",
            "Irene Mihalic",
            "Emilia Fischer",
            "Filiz Polat",
            "Lina Wagner",
            "Leon Becker",
            "Agnieszka Kalterer",
            "Felix Bauer",
            "Lara Schulz"
    };

    public final static EmailAddressType[] USER_EMAIL = new EmailAddressType[]{
            FactoryMethodInvoker.instantiateEmailAddress( "99Z@example.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "a@4.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "irene@wearefreedomnow.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "emilia.fischer@example.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "j877d3@example.this.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "lina.marie.wagner@example.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "0.1.2.3.4.5.6.7.8.becker@example.com" ),
            FactoryMethodInvoker.instantiateEmailAddress( "agna@here.ch" ),
            FactoryMethodInvoker.instantiateEmailAddress( "felix.bauer@example.org" ),
            FactoryMethodInvoker.instantiateEmailAddress( "lara.schulz@example.at" )
    };

    // The following array is used to create a list of personalAddresss for the users.
    // The indices are coded into the house number.
    // - Persons 0 - 5 are used for proximity tests with one shipment unit.
    public final static PersonalAddressType[] USER_ADDRESS = new PersonalAddressType[]{
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Marktstraße 0", "Viertelstadt",
                    FactoryMethodInvoker.instantiatePostalCode( "02314" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Hauptstraße 1", "Viertelstadt",
                    FactoryMethodInvoker.instantiatePostalCode( "02368" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Kirchplatz 2", "Niemandstown",
                    FactoryMethodInvoker.instantiatePostalCode( "12345" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Schulstraße 3", "Geisterhausen",
                    FactoryMethodInvoker.instantiatePostalCode( "31463" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Rosenweg 4", "Kuhhausen",
                    FactoryMethodInvoker.instantiatePostalCode( "72162" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Wiesenstraße 5", "Waldschenkensdorf",
                    FactoryMethodInvoker.instantiatePostalCode( "82195" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Mühlenweg 6", "Köln",
                    FactoryMethodInvoker.instantiatePostalCode( "50667" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Goethestraße 7", "Frankfurt am Main",
                    FactoryMethodInvoker.instantiatePostalCode( "60311" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Dorfstraße 8", "Stuttgart",
                    FactoryMethodInvoker.instantiatePostalCode( "70173" ) ),
            FactoryMethodInvoker.instantiatePersonalAddress(
                    "Bahnhofstraße 9", "München",
                    FactoryMethodInvoker.instantiatePostalCode( "80331" ) )
    };

    public final static UserType[] mockUsers;

    static {
        mockUsers = new UserType[USER_NAME.length];
        for ( int i = 0; i < USER_NAME.length; i++ ) {
            mockUsers[i] = new MockUser(
                    USER_NAME[i], USER_EMAIL[i], USER_ADDRESS[i] );
        }
        // TODO - remove and replace by current _norepo, in M3
        assertThrows( ShopException.class, () -> {
            new MockUser( "No one",
                    FactoryMethodInvoker.instantiateEmailAddress( null ),
                    USER_ADDRESS[0] ); }, "Check your email address validation!"  );
        assertThrows( ShopException.class, () -> {
            new MockUser( "No one",
                    FactoryMethodInvoker.instantiateEmailAddress( "Max..Gideon.Hammer@example.com" ),
                    USER_ADDRESS[0] ); }, "Check your email address validation!" );
        assertThrows( ShopException.class, () -> {
            new MockUser( "No one",
                    FactoryMethodInvoker.instantiateEmailAddress( "test@example.42" ),
                    USER_ADDRESS[0] ); }, "Check your email address validation!"  );
        assertThrows( ShopException.class, () -> {
            new MockUser( "No one",
                    FactoryMethodInvoker.instantiateEmailAddress( "test@" ),
                    USER_ADDRESS[0] ); }, "Check your email address validation!"  );
    }


    public void registerAllUsers() {
        log.info( "Registering users.");
        for ( int i = 0; i < USER_NAME.length; i++ ) {
            registerUser( USER_NAME[i], USER_EMAIL[i], USER_ADDRESS[i] );
        }
    }

    public void registerUser( String name, EmailAddressType emailAddress, PersonalAddressType personalAddress ) {
        log.info( "Registering user with email " + emailAddress );
        userRegistrationUseCases.register( name, emailAddress, personalAddress );
    }
}
