package thkoeln.archilab.ecommerce.test.regression;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.masterdata.ProductAndStockMasterDataInitializer;
import thkoeln.archilab.ecommerce.usecases.masterdata.FactoryMethodInvoker;
import thkoeln.archilab.ecommerce.usecases.masterdata.Purgatory;
import thkoeln.archilab.ecommerce.usecases.ProductCatalogUseCases;
import thkoeln.archilab.ecommerce.usecases.DepotUseCases;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;
import static thkoeln.archilab.ecommerce.usecases.masterdata.ProductAndStockMasterDataInitializer.PRODUCT_DATA;


/**
 * This test checks the management and lifecycle of depots.
 */
@SpringBootTest
public class DepotManagementTest {
    @Autowired
    private DepotUseCases depotUseCases;
    @Autowired
    private ProductCatalogUseCases productCatalogUseCases;

    @Autowired
    private Purgatory purgatory;

    private ProductAndStockMasterDataInitializer productAndStockMasterDataInitializer;

    private PersonalAddressType personalAddress1, personalAddress2;

    @BeforeEach
    public void setUp() {
        purgatory.deleteEverything();

        productAndStockMasterDataInitializer = new ProductAndStockMasterDataInitializer(
                productCatalogUseCases, depotUseCases );
        productAndStockMasterDataInitializer.addAllProducts();

        personalAddress1 = FactoryMethodInvoker.instantiatePersonalAddress(
                "Hauptstraße 1", "Berlin",
                FactoryMethodInvoker.instantiatePostalCode( "10115" ) );
        personalAddress2 = FactoryMethodInvoker.instantiatePersonalAddress(
                "Kirchplatz 2", "Hamburg",
                FactoryMethodInvoker.instantiatePostalCode( "20095" ) );
    }

    @Test
    public void testAddInvalidDepot() {
        // given
        // when
        // then
        assertThrows( ShopException.class,
                () -> depotUseCases.addNewDepot( null, null ) );
        assertThrows( ShopException.class,
                () -> depotUseCases.addNewDepot( personalAddress1, null ) );
        assertThrows( ShopException.class,
                () -> depotUseCases.addNewDepot( personalAddress1, "" ) );
    }


    @Test
    public void testDeleteAllDepots() {
        // given
        UUID id1 = depotUseCases.addNewDepot( personalAddress1, "Lager1" );
        UUID id2 = depotUseCases.addNewDepot( personalAddress2, "Lager2" );
        UUID productId9 = (UUID) PRODUCT_DATA[9][0];

        // when
        int numOf1 = depotUseCases.getAvailableStock( productId9 );
        int numOf2 = depotUseCases.getAvailableStock( productId9 );

        // then
        assertNotEquals( id1, id2 );
        assertEquals( 0, numOf1 );
        assertEquals( 0, numOf2 );
        depotUseCases.deleteAllDepots();
        // ... ids must be invalid now
        assertThrows( ShopException.class,
                () -> depotUseCases.getAvailableStock( id1 ) );
        assertThrows( ShopException.class,
                () -> depotUseCases.getAvailableStock( id2 ) );
    }
}
