package thkoeln.archilab.ecommerce.test.regression;

import com.tngtech.archunit.junit.AnalyzeClasses;
import com.tngtech.archunit.junit.ArchTag;
import com.tngtech.archunit.junit.ArchTest;
import com.tngtech.archunit.lang.ArchRule;

import static com.tngtech.archunit.lang.syntax.ArchRuleDefinition.classes;

@ArchTag("domainPrimitivesUsageRules")
@AnalyzeClasses(packages = "thkoeln.archilab.ecommerce.solution")
@SuppressWarnings("PMD")
public class DomainPrimitivesUsageRulesTest {
    private static final String EMAIL_ADDRESS_CLASS = "thkoeln.archilab.ecommerce.domainprimitives.EmailAddress";
    private static final String MONEY_CLASS = "thkoeln.archilab.ecommerce.domainprimitives.Money";
    private static final String PersonalAddress_CLASS = "thkoeln.archilab.ecommerce.domainprimitives.PersonalAddress";

    @ArchTest
    static final ArchRule userShouldReferenceEmailAddress =
            classes()
                    .that().haveSimpleName( "User" )
                    .should().dependOnClassesThat().haveFullyQualifiedName( EMAIL_ADDRESS_CLASS )
                    .because( "User should use EmailAddress instead of String" );

    @ArchTest
    static final ArchRule userShouldReferencePersonalAddress =
            classes()
                    .that().haveSimpleName( "User" )
                    .should().dependOnClassesThat().haveFullyQualifiedName( PersonalAddress_CLASS )
                    .because( "User should use PersonalAddress instead of whatever else" );

    @ArchTest
    static final ArchRule depotShouldReferenceEmailAddress =
            classes()
                    .that().haveSimpleName( "Depot" )
                    .should().dependOnClassesThat().haveFullyQualifiedName( PersonalAddress_CLASS )
                    .because( "Depot should use PersonalAddress instead of whatever else" );

    @ArchTest
    static final ArchRule productShouldReferenceMoney =
            classes()
                    .that().haveSimpleName( "Product" )
                    .should().dependOnClassesThat().haveFullyQualifiedName( MONEY_CLASS )
                    .because( "Product should use Money instead of float or whatever else" );
}
