package thkoeln.archilab.ecommerce.test.regression;

import org.junit.jupiter.api.Test;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.masterdata.FactoryMethodInvoker;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;

import static org.junit.jupiter.api.Assertions.*;
import static thkoeln.archilab.ecommerce.usecases.masterdata.FactoryMethodInvoker.instantiateEmailAddress;

class EmailAddressTest {

    @Test
    public void testToStringEmailAddress() {
        // given
        String input = "test@example.com";

        // when
        EmailAddressType instance = instantiateEmailAddress( input );

        // then
        assertEquals( input, instance.toString() );
    }

    @Test
    public void testEqualityEmailAddress() {
        // given
        // when
        EmailAddressType instance1 = instantiateEmailAddress( "test@example.com" );
        EmailAddressType instance2 = instantiateEmailAddress( "test@example.com" );
        EmailAddressType instance3 = instantiateEmailAddress( "different@example.com" );

        // then
        assertEquals( instance1, instance2 );
        assertNotEquals( instance1, instance3 );
    }

    @Test
    public void testImmutabilityEmailAddress() {
        // given
        // when
        EmailAddressType instance = instantiateEmailAddress( "test@example.com" );

        // then
        try {
            instance.getClass().getMethod( "setEmailAddress", String.class );
            fail( "setEmailAddress method should not exist" );
        } catch (NoSuchMethodException e) {
            // Success: the object is immutable
        }
    }

    @Test
    public void testFactoryValidEmailAddress() {
        // given
        // when
        // then
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "99Z@example.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "GGGhh@s77.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "a@4.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "Max.Hammer@example.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "Max.Gideon.Hammer@example.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.this.com" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.de" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.at" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.ch" ) );
        assertDoesNotThrow( () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.org" ) );
    }

    @Test
    public void testFactoryInvalidEmailAddress() {
        // given
        // when
        // then
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( null ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "testexample.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "@example.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "Max..Gideon.Hammer@example.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@examplecom" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example..com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example@that.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test example@that.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test#example@that.com" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.biz" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.biz" ) );
        assertThrows( ShopException.class, () -> FactoryMethodInvoker.instantiateEmailAddress( "test@example.42" ) );
    }

}
