package thkoeln.archilab.ecommerce.test.regression;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import thkoeln.archilab.ecommerce.usecases.masterdata.ProductAndStockMasterDataInitializer;
import thkoeln.archilab.ecommerce.usecases.masterdata.UserMasterDataInitializer;
import thkoeln.archilab.ecommerce.usecases.masterdata.Purgatory;
import thkoeln.archilab.ecommerce.usecases.*;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;

import java.util.Map;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;
import static thkoeln.archilab.ecommerce.usecases.masterdata.UserMasterDataInitializer.USER_EMAIL;


@SpringBootTest
public class OrderTest {
    @Autowired
    private UserRegistrationUseCases userRegistrationUseCases;
    @Autowired
    private ShoppingCartUseCases shoppingCartUseCases;
    @Autowired
    private ProductCatalogUseCases productCatalogUseCases;
    @Autowired
    private DepotUseCases depotUseCases;
    @Autowired
    private OrderUseCases orderUseCases;

    @Autowired
    private Purgatory purgatory;

    private UserMasterDataInitializer userMasterDataInitializer;
    private ProductAndStockMasterDataInitializer productAndStockMasterDataInitializer;


    @BeforeEach
    public void setUp() {
        purgatory.deleteEverything();

        userMasterDataInitializer = new UserMasterDataInitializer( userRegistrationUseCases );
        userMasterDataInitializer.registerAllUsers();

        productAndStockMasterDataInitializer = new ProductAndStockMasterDataInitializer(
                productCatalogUseCases, depotUseCases );
        productAndStockMasterDataInitializer.addAllProducts();
        productAndStockMasterDataInitializer.addAllDepots();
        productAndStockMasterDataInitializer.addAllStock();
    }


    @Test
    public void testorderHistory() {
        // given
        UUID productId1 = (UUID) productAndStockMasterDataInitializer.PRODUCT_DATA[1][0];
        UUID productId2 = (UUID) productAndStockMasterDataInitializer.PRODUCT_DATA[2][0];
        EmailAddressType userEmailAddress = USER_EMAIL[7];
        Map<UUID, Integer> orderHistoryBefore = orderUseCases.getOrderHistory( userEmailAddress );

        // when
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId1, 3 );
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId2, 2 );
        shoppingCartUseCases.checkout( userEmailAddress );
        Map<UUID, Integer> orderHistory1 = orderUseCases.getOrderHistory( userEmailAddress );
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId1, 6 );
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId2, 2 );
        shoppingCartUseCases.checkout( userEmailAddress );
        Map<UUID, Integer> orderHistory2 = orderUseCases.getOrderHistory( userEmailAddress );
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId1, 1 );
        shoppingCartUseCases.addProductToShoppingCart( userEmailAddress, productId2, 6 );
        shoppingCartUseCases.checkout( userEmailAddress );
        Map<UUID, Integer> orderHistory3 = orderUseCases.getOrderHistory( userEmailAddress );

        // then
        assertEquals( 0, orderHistoryBefore.size() );
        assertEquals( 2, orderHistory1.size() );
        assertEquals( 2, orderHistory2.size() );
        assertEquals( 2, orderHistory3.size() );
        assertEquals( 3, orderHistory1.get( productId1 ) );
        assertEquals( 2, orderHistory1.get( productId2 ) );
        assertEquals( 9, orderHistory2.get( productId1 ) );
        assertEquals( 4, orderHistory2.get( productId2 ) );
        assertEquals( 10, orderHistory3.get( productId1 ) );
        assertEquals( 10, orderHistory3.get( productId2 ) );
    }


    @Test
    public void testForEmptyOrderHistory() {
        // given
        EmailAddressType userEmailAddress4 = USER_EMAIL[4];

        // when
        // then
        Map<UUID, Integer> orderHistory = orderUseCases.getOrderHistory(
                userEmailAddress4 );
        assertEquals( 0, orderHistory.size() );
    }

}
