package thkoeln.archilab.ecommerce.test.regression;

import org.junit.jupiter.api.Test;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PersonalAddressType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.PostalCodeType;

import static org.junit.jupiter.api.Assertions.*;
import static thkoeln.archilab.ecommerce.usecases.masterdata.FactoryMethodInvoker.*;

class PersonalAddressTest {

    @Test
    public void testGetterPersonalAddress() {
        // given
        String street = "Irgendeinestraße 42";
        String city = "Irgendeinestadt";
        PostalCodeType plz = instantiatePostalCode( "12345" );

        // when
        PersonalAddressType instance = instantiatePersonalAddress( street, city, plz );

        // then
        assertEquals( street, instance.getStreet() );
        assertEquals( city, instance.getCity() );
        assertEquals( plz, instance.getPostalCode() );
    }

    @Test
    public void testFactoryValidPersonalAddress() {
        // given
        String street = "Irgendeinestraße 42";
        String city = "Irgendeinestadt";
        PostalCodeType plz = instantiatePostalCode( "12345" );

        // when
        // then
        assertDoesNotThrow( () -> instantiatePersonalAddress( street, city, plz ) );
    }

    @Test
    public void testFactoryInvalidPersonalAddress() {
        // given
        String street = "Irgendeinestraße 42";
        String city = "Irgendeinestadt";
        PostalCodeType plz = instantiatePostalCode( "12345" );

        // when
        // then
        assertThrows( ShopException.class, () -> instantiatePersonalAddress( null, city, plz ) );
        assertThrows( ShopException.class, () -> instantiatePersonalAddress( "", city, plz ) );
        assertThrows( ShopException.class, () -> instantiatePersonalAddress( street, null, plz ) );
        assertThrows( ShopException.class, () -> instantiatePersonalAddress( street, "", plz ) );
        assertThrows( ShopException.class, () -> instantiatePersonalAddress( street, city, null ) );
    }

    @Test
    public void testEqualityPersonalAddress() {
        // given
        String street = "Irgendeinestraße 42";
        String city = "Irgendeinestadt";
        PostalCodeType plz = instantiatePostalCode( "12345" );

        // when
        PersonalAddressType instance1 = instantiatePersonalAddress( street, city, plz );
        PersonalAddressType instance2 = instantiatePersonalAddress( street, city, plz );
        PersonalAddressType instance3 = instantiatePersonalAddress( "Anderestr. 12", city, plz );
        PersonalAddressType instance4 = instantiatePersonalAddress( street, "AndereStadt", plz );
        PersonalAddressType instance5 = instantiatePersonalAddress( street, city, instantiatePostalCode( "54321" ) );

        // then
        assertEquals( instance1, instance2 );
        assertNotEquals( instance1, instance3 );
        assertNotEquals( instance1, instance4 );
        assertNotEquals( instance1, instance5 );
    }

    @Test
    public void testImmutabilityPersonalAddress() {
        // given
        String street = "Irgendeinestraße 42";
        String city = "Irgendeinestadt";
        PostalCodeType plz = instantiatePostalCode( "12345" );

        // when
        PersonalAddressType instance = instantiatePersonalAddress( street, city, plz );

        // then
        try {
            instance.getClass().getMethod( "setStreet", String.class );
            fail( "setStreet method should not exist" );
        } catch (NoSuchMethodException e) {
            // Success: the object is immutable
        }
        try {
            instance.getClass().getMethod( "setCity", String.class );
            fail( "setCity method should not exist" );

        } catch (NoSuchMethodException e) {
            // Success: the object is immutable
        }
        try {
            instance.getClass().getMethod( "setPostalCode", PostalCodeType.class );
            fail( "setPostalCode method should not exist" );
        } catch (NoSuchMethodException e) {
            // Success: the object is immutable
        }
    }
}
