package thkoeln.archilab.ecommerce.test.regression;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import thkoeln.archilab.ecommerce.ShopException;
import thkoeln.archilab.ecommerce.usecases.masterdata.UserMasterDataInitializer;
import thkoeln.archilab.ecommerce.usecases.masterdata.Purgatory;
import thkoeln.archilab.ecommerce.usecases.UserRegistrationUseCases;
import thkoeln.archilab.ecommerce.usecases.UserType;
import thkoeln.archilab.ecommerce.usecases.domainprimitivetypes.EmailAddressType;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static thkoeln.archilab.ecommerce.usecases.masterdata.UserMasterDataInitializer.*;
import static thkoeln.archilab.ecommerce.usecases.masterdata.FactoryMethodInvoker.instantiateEmailAddress;

@SpringBootTest
public class UserRegistrationTest {
    @Autowired
    private UserRegistrationUseCases userRegistrationUseCases;
    @Autowired
    private Purgatory purgatory;

    private EmailAddressType nonExistingEmailAddress;
    private UserMasterDataInitializer userMasterDataInitializer;

    @BeforeEach
    public void setUp() {
        purgatory.deleteEverything();

        userMasterDataInitializer = new UserMasterDataInitializer( userRegistrationUseCases );
        nonExistingEmailAddress = instantiateEmailAddress( "this@nononono.de" );
    }



    @Test
    public void testRegisterUserWithDuplicateEmailAddress() {
        // given
        userMasterDataInitializer.registerAllUsers();

        // when
        // then
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.register( "Gandalf The Grey", USER_EMAIL[5],
                        USER_ADDRESS[5] ) );
    }

    @Test
    public void testRegisterUserWithDuplicateNameOrPersonalAddress() {
        // given
        userMasterDataInitializer.registerAllUsers();
        EmailAddressType newEmailAddress = instantiateEmailAddress( "some@this.de" );

        // when
        // then
        assertDoesNotThrow( () ->
                userRegistrationUseCases.register( USER_NAME[2], newEmailAddress,
                        USER_ADDRESS[2] ) );
    }


    @Test
    public void testRegisterUserWithInvalidData() {
        // given
        // when
        // then
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.register( null, USER_EMAIL[5],
                        USER_ADDRESS[5] ) );
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.register( "", USER_EMAIL[5],
                        USER_ADDRESS[5] ) );
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.register( USER_NAME[5], null,
                        USER_ADDRESS[5] ) );
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.register( USER_NAME[5], USER_EMAIL[5],
                        null ) );
    }


    @Test
    public void testChangeAddressWithInvalidData() {
        // given
        userMasterDataInitializer.registerAllUsers();

        // when
        // then
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.changeAddress( nonExistingEmailAddress, USER_ADDRESS[7] ) );
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.changeAddress( null, USER_ADDRESS[7] ) );
        assertThrows( ShopException.class, () ->
                userRegistrationUseCases.changeAddress( USER_EMAIL[5], null ) );
    }



    @Test
    public void testDeleteUsersNoMoreUsers() {
        // given
        userMasterDataInitializer.registerAllUsers();

        // when
        userRegistrationUseCases.deleteAllUsers();

        // then
        assertThrows( ShopException.class, () -> userRegistrationUseCases.getUserData( USER_EMAIL[0] ) );
    }
}
